from jsonrpc.response import JSONRPCError


class JSONRPCParseError(JSONRPCError):
    """ Parse Error.

    Invalid JSON was received by the server.
    An error occurred on the server while parsing the JSON text.
    """

    def __init__(self, **kwargs):
        super().__init__(-32700, "Parse error", **kwargs)


class JSONRPCInvalidRequest(JSONRPCError):
    """ Invalid Request.

    The JSON sent is not a valid Request object.
    """

    def __init__(self, **kwargs):
        super().__init__(-32600, "Invalid Request", **kwargs)


class JSONRPCMethodNotFound(JSONRPCError):
    """ Method not found.

    The method does not exist / is not available.
    """

    def __init__(self, **kwargs):
        super().__init__(-32601, "Method not found", **kwargs)


class JSONRPCInvalidParams(JSONRPCError):
    """ Invalid params.

    Invalid method parameter(s).
    """

    def __init__(self, **kwargs):
        super().__init__(-32602, "Invalid params", **kwargs)


class JSONRPCInternalError(JSONRPCError):
    """ Internal error.

    Internal JSON-RPC error.
    """

    def __init__(self, **kwargs):
        super().__init__(-32603, "Internal error", **kwargs)


class JSONRPCServerError(JSONRPCError):
    """ Server error.

    Reserved for implementation-defined server-errors.
    """

    def __init__(self, **kwargs):
        super().__init__(-32000, "Server error", **kwargs)



import collections


class Dispatcher(collections.MutableMapping):
    """
    Method dispatcher.
    Dictionary-like object which holds map method_name to method.
    """

    def __init__(self, prototype=None):
        """
        Build method dispatcher.

        :param prototype: Initial method mapping.
        :type prototype: None or object or dict
        """
        self.method_map = {}

        if prototype is not None:
            self.build_method_map(prototype)

    def __getitem__(self, key):
        return self.method_map[key]

    def __setitem__(self, key, value):
        self.method_map[key] = value

    def __delitem__(self, key):
        del self.method_map[key]

    def __len__(self):
        return len(self.method_map)

    def __iter__(self):
        return iter(self.method_map)

    def __repr__(self):
        return repr(self.method_map)

    def add_method(self, f, name=None):
        """
        Add a method to the dispatcher.
        When used as a decorator keep callable object unmodified.

        :param f: Callable to be added.
        :param name: Name to register
        :type f: callable
        :type name: None or str
        """
        self.method_map[name or f.__name__] = f
        return f

    def build_method_map(self, prototype):
        """
        Add prototype methods to the dispatcher.

        If given prototype is a dictionary then all callable objects will be added to dispatcher.
        If given prototype is an object then all public methods will be used.

        :param prototype: Method mapping.
        :type prototype: None or object or dict
        """
        if not isinstance(prototype, dict):
            prototype = {
                method: getattr(prototype, method)
                for method in dir(prototype)
                if not method.startswith('_')
            }

        for attr, method in prototype.items():
            if callable(method):
                self[attr] = method



from jsonrpc.dispatcher import Dispatcher
import unittest


class TestDispatcher(unittest.TestCase):

    """ Test Dispatcher functionality."""

    def setUp(self):
        self.d = Dispatcher()

    def test_getter(self):

        with self.assertRaises(KeyError):
            print(self.d["method"])

        self.d["add"] = lambda *args: sum(args)
        self.assertEqual(self.d["add"](1, 1), 2)

    def test_len(self):
        self.d["add"] = lambda *args: sum(args)
        self.assertEqual(len(self.d), 1)

    def test_in(self):
        self.d["method"] = lambda: ""
        self.assertIn("method", self.d)

    def test_add_method(self):

        @self.d.add_method
        def add(x, y):
            return x + y

        self.assertIn("add", self.d)
        self.assertEqual(self.d["add"](1, 1), 2)

    def test_add_method_keep_function_definitions(self):

        @self.d.add_method
        def one(x):
            return x

        self.assertIsNotNone(one)

    def test_del_method(self):
        self.d["method"] = lambda: ""
        self.assertIn("method", self.d)

        del self.d["method"]
        self.assertNotIn("method", self.d)

    def test_to_dict(self):
        func = lambda: ""
        self.d["method"] = func
        self.assertEqual(dict(self.d), {"method": func})

    def test_init_from_object_instance(self):

        class Dummy():

            def one(self):
                pass

            def two(self):
                pass

        dummy = Dummy()

        d = Dispatcher(dummy)

        self.assertIn("one", d)
        self.assertIn("two", d)
        self.assertNotIn("__class__", d)

    def test_init_from_dictionary(self):

        dummy = {
            'one': lambda x: x,
            'two': lambda x: x,
        }

        d = Dispatcher(dummy)

        self.assertIn("one", d)
        self.assertIn("two", d)

    def test_dispatcher_representation(self):

        self.assertEqual('{}', repr(self.d))



""" Exmples of usage with tests.

Tests in this file represent examples taken from JSON-RPC specification.
http://www.jsonrpc.org/specification#examples

"""
import unittest
import json
from jsonrpc.manager import JSONRPCResponseManager


def isjsonequal(json1, json2):
    return json.loads(json1) == json.loads(json2)


class TestJSONRPCExamples(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.manager = JSONRPCResponseManager()

    def setUp(self):
        self.dispatcher = {
            "subtract": lambda a, b: a - b,
        }

    def test_rpc_call_with_positional_parameters(self):
        req = '{"jsonrpc": "2.0", "method": "subtract", "params": [42, 23], "id": 1}'
        response = self.manager.handle(req, self.dispatcher)
        self.assertTrue(isjsonequal(
            response.json,
            '{"jsonrpc": "2.0", "result": 19, "id": 1}'
        ))

        req = '{"jsonrpc": "2.0", "method": "subtract", "params": [23, 42], "id": 2}'
        response = self.manager.handle(req, self.dispatcher)
        self.assertTrue(isjsonequal(
            response.json,
            '{"jsonrpc": "2.0", "result": -19, "id": 2}'
        ))

    def test_rpc_call_with_named_parameters(self):
        def subtract(minuend=None, subtrahend=None):
            return minuend - subtrahend

        dispatcher = {
            "subtract": subtract,
            "sum": lambda *args: sum(args),
            "get_data": lambda: ["hello", 5],
        }

        req = '{"jsonrpc": "2.0", "method": "subtract", "params": {"subtrahend": 23, "minuend": 42}, "id": 3}'
        response = self.manager.handle(req, dispatcher)
        self.assertTrue(isjsonequal(
            response.json,
            '{"jsonrpc": "2.0", "result": 19, "id": 3}'
        ))

        req = '{"jsonrpc": "2.0", "method": "subtract", "params": {"minuend": 42, "subtrahend": 23}, "id": 4}'
        response = self.manager.handle(req, dispatcher)
        self.assertTrue(isjsonequal(
            response.json,
            '{"jsonrpc": "2.0", "result": 19, "id": 4}',
        ))

    def test_notification(self):
        req = '{"jsonrpc": "2.0", "method": "subtract", "params": {"minuend": 10, "subtrahend": 4}}'

        response = self.manager.handle(req, self.dispatcher)
        self.assertEqual(response, None)

        req = '{"jsonrpc": "2.0", "method": "foobar"}'
        response = self.manager.handle(req, self.dispatcher)
        self.assertEqual(response, None)

    def test_rpc_call_of_non_existent_method(self):
        req = '{"jsonrpc": "2.0", "method": "foobar", "id": "1"}'
        response = self.manager.handle(req, self.dispatcher)
        self.assertTrue(isjsonequal(
            response.json,
            '{"jsonrpc": "2.0", "error": {"code": -32601, "message": "Method not found"}, "id": null}'
        ))

    def test_rpc_call_with_invalid_json(self):
        req = '{"jsonrpc": "2.0", "method": "foobar, "params": "bar", "baz]'
        response = self.manager.handle(req, self.dispatcher)
        self.assertTrue(isjsonequal(
            response.json,
            '{"jsonrpc": "2.0", "error": {"code": -32700, "message": "Parse error"}, "id": null}'
        ))

    def test_rpc_call_with_invalid_request_object(self):
        req = '{"jsonrpc": "2.0", "method": 1, "params": "bar"}'
        response = self.manager.handle(req, self.dispatcher)
        self.assertTrue(isjsonequal(
            response.json,
            '{"jsonrpc": "2.0", "error": {"code": -32600, "message": "Invalid Request"}, "id": null}'
        ))

    def test_rpc_call_batch_invalid_json(self):
        req = """[
            {"jsonrpc": "2.0", "method": "sum", "params": [1,2,4], "id": "1"},
            {"jsonrpc": "2.0", "method"
        ]"""
        response = self.manager.handle(req, self.dispatcher)
        self.assertTrue(isjsonequal(
            response.json,
            '{"jsonrpc": "2.0", "error": {"code": -32700, "message": "Parse error"}, "id": null}'
        ))

    def test_rpc_call_with_an_empty_array(self):
        req = '[]'
        response = self.manager.handle(req, self.dispatcher)
        self.assertTrue(isjsonequal(
            response.json,
            '{"jsonrpc": "2.0", "error": {"code": -32600, "message": "Invalid Request"}, "id": null}'
        ))

    def test_rpc_call_with_rpc_call_with_an_invalid_batch_but_not_empty(self):
        req = '[1]'
        response = self.manager.handle(req, self.dispatcher)
        self.assertTrue(isjsonequal(
            response.json,
            '{"jsonrpc": "2.0", "error": {"code": -32600, "message": "Invalid Request"}, "id": null}'
        ))

    def test_rpc_call_with_invalid_batch(self):
        req = '[1,2,3]'
        response = self.manager.handle(req, self.dispatcher)
        self.assertTrue(
            response,
            json.loads("""[
                {"jsonrpc": "2.0", "error": {"code": -32600,
                "message": "Invalid Request"}, "id": null},
                {"jsonrpc": "2.0", "error": {"code": -32600,
                "message": "Invalid Request"}, "id": null},
                {"jsonrpc": "2.0", "error": {"code": -32600,
                "message": "Invalid Request"}, "id": null}
            ]""")
        )

    def test_rpc_call_batch(self):
        req = """[
            {"jsonrpc": "2.0", "method": "sum", "params": [1,2,4], "id": "1"},
            {"jsonrpc": "2.0", "method": "notify_hello", "params": [7]},
            {"jsonrpc": "2.0", "method": "subtract",
            "params": [42,23], "id": "2"},
            {"foo": "boo"},
            {"jsonrpc": "2.0", "method": "foo.get",
            "params": {"name": "myself"}, "id": "5"},
            {"jsonrpc": "2.0", "method": "get_data", "id": "9"}
        ]"""
        response = self.manager.handle(req, self.dispatcher)
        self.assertTrue(
            response,
            json.loads("""[
                {"jsonrpc": "2.0", "result": 7, "id": "1"},
                {"jsonrpc": "2.0", "result": 19, "id": "2"},
                {"jsonrpc": "2.0", "error": {"code": -32600,
                "message": "Invalid Request"}, "id": null},
                {"jsonrpc": "2.0", "error": {"code": -32601,
                "message": "Method not found"}, "id": "5"},
                {"jsonrpc": "2.0", "result": ["hello", 5], "id": "9"}
            ]""")
        )

    def test_rpc_call_batch_all_notifications(self):
        req = """[
            {"jsonrpc": "2.0", "method": "notify_sum", "params": [1,2,4]},
            {"jsonrpc": "2.0", "method": "notify_hello", "params": [7]}
        ]"""
        response = self.manager.handle(req, self.dispatcher)
        self.assertEqual(response, None)



import json
import unittest
from copy import deepcopy

from jsonrpc.exceptions import JSONRPCInvalidRequestException, JSONRPCParseException, JSONRPCMultipleRequestException, \
    JSONRPCException
from jsonrpc.request import JSONRPCSingleRequest, JSONRPCBatchRequest
from jsonrpc.response import JSONRPCSingleResponse, JSONRPCBatchResponse
from jsonrpc.tests.test_examples import isjsonequal


class TestJSONRPCSingleRequest(unittest.TestCase):
    """ Test JSONRPCSingleRequest functionality."""

    def setUp(self):
        self.request_params = {
            "jsonrpc": "2.0",
            "method": "add",
            "params": [1, 2],
            "id": 1,
        }

    def test_correct_init(self):
        """ Test object is created."""
        JSONRPCSingleRequest(self.request_params)

    def test_validation_incorrect_no_parameters(self):
        with self.assertRaises(TypeError):
            JSONRPCSingleRequest()

    def test_version_validation(self):
        with self.assertRaises(JSONRPCInvalidRequestException):
            JSONRPCSingleRequest({"jsonrpc": "3.0", "method": "add"})

    def test_method_validation_str(self):
        self.request_params.update({"method": "add"})
        JSONRPCSingleRequest(self.request_params)

    def test_method_validation_not_str(self):
        self.request_params.update({"method": []})
        with self.assertRaises(JSONRPCInvalidRequestException):
            JSONRPCSingleRequest(self.request_params)

        self.request_params.update({"method": {}})
        with self.assertRaises(JSONRPCInvalidRequestException):
            JSONRPCSingleRequest(self.request_params)

    def test_method_validation_str_rpc_prefix(self):
        """ Test method SHOULD NOT starts with rpc. """
        self.request_params.update({"method": "rpc."})
        with self.assertRaises(JSONRPCInvalidRequestException):
            JSONRPCSingleRequest(self.request_params)

        self.request_params.update({"method": "rpc.test"})
        with self.assertRaises(JSONRPCInvalidRequestException):
            JSONRPCSingleRequest(self.request_params)

        self.request_params.update({"method": "rpccorrect"})
        JSONRPCSingleRequest(self.request_params)

        self.request_params.update({"method": "rpc"})
        JSONRPCSingleRequest(self.request_params)

    def test_params_validation_list(self):
        self.request_params.update({"params": []})
        JSONRPCSingleRequest(self.request_params)

        self.request_params.update({"params": [0]})
        JSONRPCSingleRequest(self.request_params)

    def test_params_validation_tuple(self):
        self.request_params.update({"params": ()})
        JSONRPCSingleRequest(self.request_params)

        self.request_params.update({"params": tuple([0])})
        JSONRPCSingleRequest(self.request_params)

    def test_params_validation_dict(self):
        self.request_params.update({"params": {}})
        JSONRPCSingleRequest(self.request_params)

        self.request_params.update({"params": {"a": 0}})
        JSONRPCSingleRequest(self.request_params)

    def test_params_validation_none(self):
        self.request_params.update({"params": None})
        with self.assertRaises(JSONRPCInvalidRequestException):
            JSONRPCSingleRequest(self.request_params)

    def test_params_validation_incorrect(self):
        self.request_params.update({"params": "str"})
        with self.assertRaises(JSONRPCInvalidRequestException):
            JSONRPCSingleRequest(self.request_params)

    def test_request_args(self):
        req_data = {"jsonrpc": "2.0", "method": "add", "id": 1}
        self.assertEqual(JSONRPCSingleRequest(req_data).args, ())
        self.assertEqual(JSONRPCSingleRequest(dict(req_data, params=[])).args, ())
        self.assertEqual(JSONRPCSingleRequest(dict(req_data, params={"a": 1})).args, ())
        self.assertEqual(JSONRPCSingleRequest(dict(req_data, params=[1, 2])).args, (1, 2))

    def test_request_kwargs(self):
        req_data = {"jsonrpc": "2.0", "method": "add", "id": 1}
        self.assertEqual(JSONRPCSingleRequest(req_data).kwargs, {})
        self.assertEqual(JSONRPCSingleRequest(dict(req_data, params=[1, 2])).kwargs, {})
        self.assertEqual(JSONRPCSingleRequest(dict(req_data, params={})).kwargs, {})
        self.assertEqual(JSONRPCSingleRequest(dict(req_data, params={"a": 1})).kwargs, {"a": 1})

    def test_id_validation_string(self):
        self.request_params.update({"id": "id"})
        JSONRPCSingleRequest(self.request_params)

    def test_id_validation_int(self):
        self.request_params.update({"id": 0})
        JSONRPCSingleRequest(self.request_params)

    def test_id_validation_null(self):
        self.request_params.update({"id": "null"})
        JSONRPCSingleRequest(self.request_params)

    def test_id_validation_none(self):
        self.request_params.update({"id": None})
        with self.assertRaises(JSONRPCInvalidRequestException):
            JSONRPCSingleRequest(self.request_params)

    def test_id_validation_float(self):
        self.request_params.update({"id": 0.1})
        with self.assertRaises(JSONRPCInvalidRequestException):
            JSONRPCSingleRequest(self.request_params)

    def test_id_validation_incorrect(self):
        self.request_params.update({"id": []})
        with self.assertRaises(JSONRPCInvalidRequestException):
            JSONRPCSingleRequest(self.request_params)

        self.request_params.update({"id": ()})
        with self.assertRaises(JSONRPCInvalidRequestException):
            JSONRPCSingleRequest(self.request_params)

    def test_incorrect_multiple_json(self):
        lst = [
            {"method": "devide", "params": {"num": 1, "denom": 2}, "id": 1, "jsonrpc": "2.0"},
            {"method": "devide", "params": {"num": 3, "denom": 2}, "id": 2, "jsonrpc": "2.0"}
        ]

        with self.assertRaises(JSONRPCMultipleRequestException):
            JSONRPCSingleRequest(json.dumps(lst))

        with self.assertRaises(JSONRPCMultipleRequestException):
            JSONRPCSingleRequest(lst)

    def test_incorrect_empty_request(self):
        request = {}

        with self.assertRaises(JSONRPCInvalidRequestException):
            JSONRPCSingleRequest(json.dumps(request))

        with self.assertRaises(JSONRPCInvalidRequestException):
            JSONRPCSingleRequest(request)

    def test_data_method_1(self):
        r = JSONRPCSingleRequest({"jsonrpc": "2.0", "method": "add"})
        self.assertEqual(r.data, {
            "jsonrpc": "2.0",
            "method": "add",
        })

    def test_data_method_2(self):
        r = JSONRPCSingleRequest({"jsonrpc": "2.0", "method": "add", "id": 1})
        self.assertEqual(r.data, {
            "jsonrpc": "2.0",
            "method": "add",
            "id": 1,
        })

    def test_data_params_1(self):
        with self.assertRaises(JSONRPCInvalidRequestException):
            JSONRPCSingleRequest({"jsonrpc": "2.0", "method": "add", "params": None, "id": 1})

    def test_data_params_2(self):
        r = JSONRPCSingleRequest({"jsonrpc": "2.0", "method": "add", "params": [], "id": 1})
        self.assertEqual(r.data, {
            "jsonrpc": "2.0",
            "method": "add",
            "params": [],
            "id": 1,
        })

    def test_data_params_3(self):
        r = JSONRPCSingleRequest({"jsonrpc": "2.0", "method": "add", "params": (), "id": 1})
        self.assertEqual(r.data, {
            "jsonrpc": "2.0",
            "method": "add",
            "params": (),
            "id": 1,
        })

    def test_data_params_4(self):
        r = JSONRPCSingleRequest({"jsonrpc": "2.0", "method": "add", "params": {}, "id": 1})
        self.assertEqual(r.data, {
            "jsonrpc": "2.0",
            "method": "add",
            "params": {},
            "id": 1,
        })

    def test_data_params_5(self):
        r = JSONRPCSingleRequest({"jsonrpc": "2.0", "method": "add", "params": (1, 2), "id": 1})
        self.assertEqual(r.data, {
            "jsonrpc": "2.0",
            "method": "add",
            "params": (1, 2),
            "id": 1,
        })

    def test_data_params_6(self):
        r = JSONRPCSingleRequest({"jsonrpc": "2.0", "method": "add", "params": {"a": 0}, "id": 1})
        self.assertEqual(r.data, {
            "jsonrpc": "2.0",
            "method": "add",
            "params": {"a": 0},
            "id": 1,
        })

    def test_data_id_1(self):
        r = JSONRPCSingleRequest({"jsonrpc": "2.0", "method": "add", "id": "null"})
        self.assertEqual(r.data, {
            "jsonrpc": "2.0",
            "method": "add",
            "id": "null",
        })

    def test_data_id_1_notification(self):
        r = JSONRPCSingleRequest({"jsonrpc": "2.0", "method": "add"})
        self.assertEqual(r.data, {
            "jsonrpc": "2.0",
            "method": "add",
        })

    def test_data_id_2(self):
        r = JSONRPCSingleRequest({"jsonrpc": "2.0", "method": "add", "id": 1})
        self.assertEqual(r.data, {
            "jsonrpc": "2.0",
            "method": "add",
            "id": 1,
        })

    def test_data_id_3(self):
        r = JSONRPCSingleRequest({"jsonrpc": "2.0", "method": "add", "id": "id"})
        self.assertEqual(r.data, {
            "jsonrpc": "2.0",
            "method": "add",
            "id": "id",
        })

    def test_is_notification(self):
        r = JSONRPCSingleRequest({"jsonrpc": "2.0", "method": "add"})
        self.assertTrue(r.is_notification)

        r = JSONRPCSingleRequest({"jsonrpc": "2.0", "method": "add", "id": 1})
        self.assertFalse(r.is_notification)

        r = JSONRPCSingleRequest({"jsonrpc": "2.0", "method": "add", "id": 'null'})
        self.assertFalse(r.is_notification)

        r = JSONRPCSingleRequest({"jsonrpc": "2.0", "method": "add", "id": 0})
        self.assertFalse(r.is_notification)

        r = JSONRPCSingleRequest({"jsonrpc": "2.0", "method": "add", "id": 1})
        r.is_notification = True
        self.assertTrue(r.is_notification)
        self.assertNotIn("id", r.data)

        r = JSONRPCSingleRequest({"jsonrpc": "2.0", "method": "add", "id": 0})
        r.is_notification = True
        self.assertTrue(r.is_notification)
        self.assertNotIn("id", r.data)

    def test_set_unset_notification_keep_id(self):
        r = JSONRPCSingleRequest({"jsonrpc": "2.0", "method": "add", "id": 0})
        r.is_notification = True
        self.assertTrue(r.is_notification)
        self.assertNotIn("id", r.data)

        r.is_notification = False
        self.assertFalse(r.is_notification)
        self.assertTrue("id" in r.data)
        self.assertEqual(r.data["id"], 0)

    def test_serialize_method_1(self):
        r = JSONRPCSingleRequest({"jsonrpc": "2.0", "method": "add"})
        self.assertEqual(
            {
                "jsonrpc": "2.0",
                "method": "add",
            }, json.loads(r.json))

    def test_serialize_method_2(self):
        r = JSONRPCSingleRequest({"jsonrpc": "2.0", "method": "add", "id": 1})
        self.assertEqual(
            {
                "jsonrpc": "2.0",
                "method": "add",
                "id": 1,
            },
            json.loads(r.json)
        )

    def test_serialize_params_1(self):
        r = JSONRPCSingleRequest({"jsonrpc": "2.0", "method": "add", "params": [], "id": 1})
        self.assertEqual(
            {
                "jsonrpc": "2.0",
                "method": "add",
                "params": [],
                "id": 1,
            },
            json.loads(r.json)
        )

    def test_serialize_params_3(self):
        r = JSONRPCSingleRequest({"jsonrpc": "2.0", "method": "add", "params": (), "id": 1})
        self.assertEqual(
            {
                "jsonrpc": "2.0",
                "method": "add",
                "params": [],
                "id": 1,
            },
            json.loads(r.json)
        )

    def test_serialize_params_4(self):
        r = JSONRPCSingleRequest({"jsonrpc": "2.0", "method": "add", "params": [1, 2], "id": 1})
        self.assertEqual(
            {
                "jsonrpc": "2.0",
                "method": "add",
                "params": [1, 2],
                "id": 1,
            },
            json.loads(r.json)
        )

    def test_serialize_params_5(self):
        r = JSONRPCSingleRequest({"jsonrpc": "2.0", "method": "add", "params": {"a": 0}, "id": 1})
        self.assertEqual(
            {
                "jsonrpc": "2.0",
                "method": "add",
                "params": {"a": 0},
                "id": 1,
            },
            json.loads(r.json)
        )

    def test_serialize_id_1(self):
        r = JSONRPCSingleRequest({"jsonrpc": "2.0", "method": "add", "id": "null"})
        self.assertEqual(
            {
                "jsonrpc": "2.0",
                "method": "add",
                "id": "null",
            },
            json.loads(r.json)
        )

    def test_serialize_id_3(self):
        r = JSONRPCSingleRequest({"jsonrpc": "2.0", "method": "add", "id": "id"})
        self.assertEqual(
            {
                "jsonrpc": "2.0",
                "method": "add",
                "id": "id",
            },
            json.loads(r.json)
        )

    def test_serialize_id_4(self):
        r = JSONRPCSingleRequest({"jsonrpc": "2.0", "method": "add", "id": 0})
        self.assertEqual(
            {
                "jsonrpc": "2.0",
                "method": "add",
                "id": 0,
            },
            json.loads(r.json)
        )

    def test_from_json_request_no_id(self):
        str_json = json.dumps({
            "method": "add",
            "params": [1, 2],
            "jsonrpc": "2.0",
        })

        request = JSONRPCSingleRequest(str_json)
        self.assertIsInstance(request, JSONRPCSingleRequest)
        self.assertEqual(request.method, "add")
        self.assertEqual(request.params, [1, 2])
        self.assertEqual(request.id, None)
        self.assertTrue(request.is_notification)

    def test_from_json_request_no_params(self):
        str_json = json.dumps({
            "method": "add",
            "jsonrpc": "2.0",
        })

        request = JSONRPCSingleRequest(str_json)
        self.assertIsInstance(request, JSONRPCSingleRequest)
        self.assertEqual(request.method, "add")
        self.assertEqual(request.params, None)
        self.assertEqual(request.id, None)
        self.assertTrue(request.is_notification)

    def test_from_json_request(self):
        str_json = json.dumps({
            "method": "add",
            "params": [0, 1],
            "jsonrpc": "2.0",
            "id": "id",
        })

        request = JSONRPCSingleRequest(str_json)
        self.assertIsInstance(request, JSONRPCSingleRequest)
        self.assertEqual(request.method, "add")
        self.assertEqual(request.params, [0, 1])
        self.assertEqual(request.id, "id")
        self.assertFalse(request.is_notification)

    def test_from_json_invalid_request_jsonrpc(self):
        str_json = json.dumps({
            "method": "add",
        })

        with self.assertRaises(JSONRPCInvalidRequestException):
            JSONRPCSingleRequest(str_json)

    def test_from_json_invalid_request_method(self):
        str_json = json.dumps({
            "jsonrpc": "2.0",
        })

        with self.assertRaises(JSONRPCInvalidRequestException):
            JSONRPCSingleRequest(str_json)

    def test_from_json_invalid_request_extra_data(self):
        str_json = json.dumps({
            "jsonrpc": "2.0",
            "method": "add",
            "is_notification": True,
        })

        with self.assertRaises(JSONRPCInvalidRequestException):
            JSONRPCSingleRequest(str_json)

    def test_data_setter(self):
        request = JSONRPCSingleRequest(self.request_params)
        with self.assertRaises(JSONRPCMultipleRequestException):
            request.data = []

        with self.assertRaises(JSONRPCParseException):
            request.data = ""

        with self.assertRaises(JSONRPCInvalidRequestException):
            request.data = None


class TestJSONRPCBatchRequest(unittest.TestCase):
    """ Test JSONRPCBatchRequest functionality."""

    def test_batch_request(self):
        request = JSONRPCBatchRequest([
            JSONRPCSingleRequest({"method": "devide", "params": {"num": 1, "denom": 2}, "id": 1, "jsonrpc": "2.0"}),
            JSONRPCSingleRequest({"method": "devide", "params": {"num": 3, "denom": 2}, "id": 2, "jsonrpc": "2.0"}),
        ])
        self.assertEqual(json.loads(request.json), [
            {"method": "devide", "params": {"num": 1, "denom": 2}, "id": 1, "jsonrpc": "2.0"},
            {"method": "devide", "params": {"num": 3, "denom": 2}, "id": 2, "jsonrpc": "2.0"},
        ])
        self.assertTrue(request)

    def test_batch_request_from_list_dicts(self):
        request = JSONRPCBatchRequest([
            {"method": "devide", "params": {"num": 1, "denom": 2}, "id": 1, "jsonrpc": "2.0"},
            {"method": "devide", "params": {"num": 3, "denom": 2}, "id": 2, "jsonrpc": "2.0"},
        ])
        self.assertEqual(json.loads(request.json), [
            {"method": "devide", "params": {"num": 1, "denom": 2}, "id": 1, "jsonrpc": "2.0"},
            {"method": "devide", "params": {"num": 3, "denom": 2}, "id": 2, "jsonrpc": "2.0"},
        ])
        self.assertTrue(request)

    def test_from_json_batch(self):
        str_json = json.dumps([
            {"method": "add", "params": [1, 2], "jsonrpc": "2.0"},
            {"method": "mul", "params": [1, 2], "jsonrpc": "2.0"},
        ])

        requests = JSONRPCBatchRequest(str_json)
        self.assertIsInstance(requests, JSONRPCBatchRequest)
        for r in requests:
            self.assertIsInstance(r, JSONRPCSingleRequest)
            self.assertTrue(r.method in ["add", "mul"])
            self.assertEqual(r.params, [1, 2])
            self.assertEqual(r.id, None)
            self.assertTrue(r.is_notification)

    def test_from_json_batch_one(self):
        req_data = {"method": "add", "params": [1, 2], "jsonrpc": "2.0", "id": 1}

        request = JSONRPCSingleRequest(req_data)
        requests = JSONRPCBatchRequest(request)
        self.assertIsInstance(requests, JSONRPCBatchRequest)
        self.assertEqual(len(requests), 1)
        r = requests[0]
        self.assertIsInstance(r, JSONRPCSingleRequest)
        self.assertEqual(r.method, "add")
        self.assertEqual(r.params, [1, 2])
        self.assertEqual(r.id, 1)
        self.assertFalse(r.is_notification)

    def test_response_iterator(self):
        requests = [
            JSONRPCSingleRequest({"method": "devide", "params": {"num": 1, "denom": 2}, "id": 1, "jsonrpc": "2.0"}),
            JSONRPCSingleRequest({"method": "devide", "params": {"num": 3, "denom": 2}, "id": 2, "jsonrpc": "2.0"}),
        ]
        batch = JSONRPCBatchRequest(requests)
        for request in batch:
            self.assertIsInstance(request, JSONRPCSingleRequest)
            self.assertIsInstance(batch, JSONRPCBatchRequest)
            self.assertEqual(request.method, "devide")

    def test_incorrect_batch_request(self):
        with self.assertRaises(JSONRPCInvalidRequestException):
            JSONRPCBatchRequest(1.3)


class TestJSONRPCSingleResponse(unittest.TestCase):
    """ Test JSONRPCSingleResponse functionality."""

    @classmethod
    def setUpClass(cls):
        cls.request = JSONRPCSingleRequest({"method": "add", "params": [1, 2], "jsonrpc": "2.0", "id": 1})
        cls.result = 3
        cls.error_data = {"code": 1, "message": "error"}

    def test_correct_init(self):
        """ Test object is created."""
        JSONRPCSingleResponse(self.result, request=self.request)

    def test_validation_incorrect_no_parameters(self):
        with self.assertRaises(TypeError):
            JSONRPCSingleResponse()

    def test_validation_incorrect_result_and_error(self):
        with self.assertRaises(ValueError):
            JSONRPCSingleResponse(self.error_data, error=False)

    def test_validation_error_correct(self):
        JSONRPCSingleResponse(self.error_data, error=True)

    def test_validation_error_incorrect(self):
        d = deepcopy(self.error_data)
        d.update({"code": "str"})
        with self.assertRaises(ValueError):
            JSONRPCSingleResponse(d, error=True)

    def test_validation_error_incorrect_no_code(self):
        d = deepcopy(self.error_data)
        del d["code"]
        with self.assertRaises(ValueError):
            JSONRPCSingleResponse(d, error=True)

    def test_validation_error_incorrect_no_message(self):
        d = deepcopy(self.error_data)
        del d["message"]
        with self.assertRaises(ValueError):
            JSONRPCSingleResponse(d, error=True)

    def test_validation_error_payload_type(self):
        with self.assertRaises(TypeError):
            JSONRPCSingleResponse(None, error=True)

    def test_validation_error_incorrect_message_not_str(self):
        d = deepcopy(self.error_data)
        d.update({"message": 0})
        with self.assertRaises(ValueError):
            JSONRPCSingleResponse(d, error=True)

    def test_validation_id(self):
        response = JSONRPCSingleResponse(self.result, request=self.request)
        self.assertEqual(response.id, self.request.id)

    def test_container_result(self):
        r = JSONRPCSingleResponse(self.result, self.request)
        self.assertEqual(json.loads(r.json), r.container)
        self.assertEqual(r.container, {
            "jsonrpc": "2.0",
            "result": 3,
            "id": 1,
        })

    def test_container_error(self):
        r = JSONRPCSingleResponse(self.error_data, error=True)
        self.assertEqual(json.loads(r.json), r.container)
        self.assertEqual(r.container, {
            "jsonrpc": "2.0",
            "error": {
                "code": 1,
                "message": "error",
            },
            "id": None,
        })


class TestJSONRPCBatchResponse(unittest.TestCase):
    """ Test JSONRPCBatchResponse functionality."""

    def test_batch_response(self):
        response = JSONRPCBatchResponse([
            JSONRPCSingleResponse("pong", request=JSONRPCSingleRequest({"method": "ping", "jsonrpc": "2.0", "id": 1})),
            JSONRPCSingleResponse({"code": 0, "message": ""}, error=True),
        ])
        self.assertEqual(json.loads(response.json), [
            {"result": "pong", "id": 1, "jsonrpc": "2.0"},
            {"error": {"code": 0, "message": ""}, "id": None, "jsonrpc": "2.0"},
        ])

    def test_batch_response_from_single(self):
        request = JSONRPCSingleRequest({"method": "ping", "jsonrpc": "2.0", "id": 1})
        result = "pong"
        response = JSONRPCSingleResponse(result, request=request)

        batch_with_list = JSONRPCBatchResponse([response])
        batch_without_list = JSONRPCBatchResponse(response)

        self.assertTrue(isjsonequal(batch_with_list.json, batch_without_list.json))

        iter(batch_with_list)
        iter(batch_without_list)
        iter(response)

        self.assertEqual(len(batch_with_list), 1)
        self.assertEqual(len(batch_without_list), 1)
        self.assertTrue(isjsonequal(response.json, batch_with_list[0].json))
        self.assertTrue(isjsonequal(response.json, batch_without_list[0].json))

    def test_incorrect_constructor(self):
        with self.assertRaises(TypeError):
            JSONRPCBatchResponse(1)
        with self.assertRaises(TypeError):
            JSONRPCBatchResponse([1, 2])
        with self.assertRaises(JSONRPCException):
            JSONRPCBatchResponse(None)

    def test_response_iterator(self):
        request_1 = JSONRPCSingleRequest({"method": "add", "params": [1, 2], "jsonrpc": "2.0", "id": 1})
        request_2 = JSONRPCSingleRequest({"method": "mul", "params": [7, 3], "jsonrpc": "2.0", "id": 2})
        request_3 = JSONRPCSingleRequest({"method": "ping", "jsonrpc": "2.0"})
        result_1 = 2
        result_2 = 21
        result_3 = "pong"
        responses = JSONRPCBatchResponse([
            JSONRPCSingleResponse(result_1, request=request_1),
            JSONRPCSingleResponse(result_2, request=request_2),
            JSONRPCSingleResponse(result_3, request=request_3)
        ])

        for response in responses:
            self.assertIsInstance(response, JSONRPCSingleResponse)



import json

import requests


url = "http://localhost:4000/jsonrpc"
headers = {'content-type': 'application/json'}


def print_result(payload):
    response = requests.post(
        url,
        data=json.dumps(payload),
        headers=headers
    ).json()

    print(
        r"""
        {0}
        {1}
        """.format(payload, response))


def main():
    payloads = [
        {
            "method": "simple_add",
            "params": {"first": 17, "second": 39},
            "jsonrpc": "2.0",
            "id": 0,
        },
        {
            "method": "echo",
            "params": ["Hello!"],
            "jsonrpc": "2.0",
            "id": 1
        },
        {
            "method": "dict_to_list",
            "jsonrpc": "2.0",
            "params": [{1: 3, 'two': 'string', 3: [5, 'list', {'c': 0.3}]}],
            "id": 3
        },
        # Exception!
        {
            "method": "subtract",
            "jsonrpc": "2.0",
            "params": [1, 2, 3],
            "id": 2
        }
    ]
    for payload in payloads:
        print_result(payload)


if __name__ == "__main__":
    main()



""" JSON-RPC response wrappers """
from jsonrpc.base import JSONSerializable
from jsonrpc.exceptions import JSONRPCException


class JSONRPCError(JSONSerializable):
    """ Error for JSON-RPC communication.

    The error codes from and including -32768 to -32000 are reserved for
    pre-defined errors. Any code within this range, but not defined explicitly
    below is reserved for future use. The error codes are nearly the same as
    those suggested for XML-RPC at the following
    url: http://xmlrpc-epi.sourceforge.net/specs/rfc.fault_codes.php
    """

    def __init__(self, code, message, data=None, serialize_hook=None, deserialize_hook=None):
        """
        When a rpc call encounters an error, the Response Object MUST contain the
        error member with a value that is a Object with the following members in __init__

        :param int code: A Number that indicates the error type that occurred.
            This MUST be an integer.
        :param str message: A String providing a short description of the error.
            The message SHOULD be limited to a concise single sentence.
        :param data: A Primitive or Structured value that contains additional
            information about the error.
            This may be omitted.
            The value of this member is defined by the Server (e.g. detailed error
            information, nested errors etc.).
        :type data: None or int or str or dict or list
        """

        super().__init__(serialize_hook=serialize_hook, deserialize_hook=deserialize_hook)
        self._container = {}
        if not isinstance(code, int):
            raise ValueError("Error code should be integer")
        else:
            self._container['code'] = code

        if not isinstance(message, str):
            raise ValueError("Error message should be string")
        else:
            self._container['message'] = message

        if data is not None:
            self._container['data'] = data

    @property
    def code(self):
        return self._container['code']

    @property
    def message(self):
        return self._container['message']

    @property
    def data(self):
        return self._container.get('data')

    @property
    def json(self):
        return self.serialize(self._container)

    def as_response(self):
        return JSONRPCSingleResponse(payload=self._container, error=True)


class JSONRPCSingleResponse(JSONSerializable):
    """ JSON-RPC response object to JSONRPCRequest. """
    _error_flag = None
    _payload = None
    _request = None

    def __init__(self, payload, request=None, error=None, serialize_hook=None, deserialize_hook=None):
        """
        :param request: Bound request for response
        :type request: JSONRPCSingleRequest
        :param payload: Result or error dict
        :param error: Error Flag
        :type error: bool
        """
        super().__init__(serialize_hook=serialize_hook, deserialize_hook=deserialize_hook)
        if not error:
            if request is None:
                raise ValueError("Can't create non-error Response object without request object!")
        else:
            if not isinstance(payload, dict):
                raise TypeError("Error payload should be dict, not {0}".format(type(payload)))
            if 'code' not in payload or 'message' not in payload:
                raise ValueError("Can't find code and/or message in payload")
            if not isinstance(payload['code'], int):
                raise ValueError("Error code should be integer")
            if not isinstance(payload['message'], str):
                raise ValueError("Error message should be string")

        self._request = request
        self._payload = payload
        self._error_flag = error

    def __iter__(self):
        yield self

    @property
    def result(self):
        return self._payload if not self._error_flag else None

    @property
    def error(self):
        return self._payload if self._error_flag else None

    @property
    def id(self):
        return self._request.id if not self._error_flag else None

    @property
    def container(self):
        data = {"jsonrpc": "2.0", "id": self.id}
        if self._error_flag:
            data["error"] = self.error
        else:
            data["result"] = self.result
        return data

    @property
    def json(self):
        return self.serialize(self.container)


class JSONRPCBatchResponse(JSONSerializable):
    _data = []

    def __iter__(self):
        return iter(self._data)

    def __len__(self):
        return len(self._data)

    def __getitem__(self, item):
        return self._data.__getitem__(item)

    @property
    def json(self):
        return self.serialize([response.container for response in self])

    def __init__(self, response, serialize_hook=None):
        super().__init__(serialize_hook=serialize_hook)
        self._data = self._validate(response)

    def _validate(self, raw_data):
        self._valid_flag = False
        data = []
        if not raw_data:
            raise JSONRPCException("Empty batch response data!")

        if isinstance(raw_data, (list, tuple, JSONRPCSingleResponse)):
            for item in raw_data:
                if isinstance(item, JSONRPCSingleResponse):
                    data.append(item)
                else:
                    raise TypeError(
                        "Response item must be JSONRPCSingleResponse instance, not {0}"
                        .format(type(raw_data))
                    )
        else:
            raise TypeError(
                "Responses must be list, tuple or JSONRPCSingleResponse, not {0}"
                .format(type(raw_data))
            )
        self._valid_flag = True
        return data



from functools import partial
from json import loads, dumps


class JSONSerializable:
    """ Common functionality for json serializable objects.
    Provides support for custom json serialization/deserialization hooks
    via serialize_hook — dumps(default=…) and deserialize_hook — loads(object_hook=…)
    """

    def __init__(self, serialize_hook=None, deserialize_hook=None):
        self.serialize = partial(dumps, default=serialize_hook)
        self.deserialize = partial(loads, object_hook=deserialize_hook)
        self.serialize_hook = serialize_hook
        self.deserialize_hook = deserialize_hook



class JSONRPCException(Exception):
    """ JSON-RPC Exception."""
    pass


class JSONRPCParseException(Exception):
    """ Can't parse request from string."""
    pass


class JSONRPCMultipleRequestException(Exception):
    """ Found multiple requests. Try use batch instead"""
    pass


class JSONRPCInvalidRequestException(JSONRPCException):
    """ Request is not valid."""
    pass



""" Test utility functionality."""
import unittest

from jsonrpc.base import JSONSerializable


class TestJSONSerializable(unittest.TestCase):
    """ Test JSONSerializable functionality."""

    def setUp(self):
        self._class = JSONSerializable

    def test_definse_serialize_deserialize(self):
        """ Test classmethods of inherited class."""
        self.assertEqual(self._class().serialize({}), "{}")
        self.assertEqual(self._class().deserialize("{}"), {})



from jsonrpc.errors import JSONRPCInvalidRequest, JSONRPCParseError
from jsonrpc.exceptions import JSONRPCInvalidRequestException, JSONRPCParseException
from jsonrpc.request import JSONRPCSingleRequest, JSONRPCBatchRequest
from jsonrpc.response import JSONRPCSingleResponse
from jsonrpc.base import JSONSerializable


class JSONRPCResponseManager(JSONSerializable):
    """ JSON-RPC response manager. """

    def handle(self, request_string, dispatcher):
        """
        Method brings syntactic sugar into library.
        Given dispatcher it handles request (both single and batch) and handles errors.
        Request could be handled in parallel, it is server responsibility.

        :param request_string: JSON string.
            Will be converted into JSONRPCSingleRequest or JSONRPCBatchRequest
        :type request_string: str
        :type dispatcher: Dispatcher or dict
        :rtype: JSONRPCSingleResponse or JSONRPCBatchResponse
        """

        try:
            data = self.deserialize(request_string)
            if isinstance(data, list):
                request = JSONRPCBatchRequest(data, serialize_hook=self.serialize_hook)
            elif isinstance(data, dict):
                request = JSONRPCSingleRequest(data, serialize_hook=self.serialize_hook)
            else:
                raise JSONRPCInvalidRequestException
        except (TypeError, ValueError, JSONRPCParseException):
            return JSONRPCParseError().as_response()
        except JSONRPCInvalidRequestException:
            return JSONRPCInvalidRequest().as_response()
        else:
            return request.process(dispatcher)



""" JSON-RPC request wrappers """
from jsonrpc.base import JSONSerializable
from jsonrpc.response import JSONRPCBatchResponse, JSONRPCSingleResponse
from jsonrpc.exceptions import JSONRPCParseException, JSONRPCMultipleRequestException, JSONRPCInvalidRequestException
from jsonrpc.errors import JSONRPCMethodNotFound, JSONRPCInvalidParams, JSONRPCServerError


class JSONRPCBaseRequest(JSONSerializable):
    """ Base wrapper class for JSON-RPC requests
    :param _data: Dictionary with internal data
    :type _data: dict
    :param _valid_flag: Internal flag. True, if request is valid. Used by __bool__()
    :type _valid_flag: bool
    """

    _data = None
    _valid_flag = False

    def __init__(self, request, serialize_hook=None, deserialize_hook=None):
        """ Initialize request and validate input data"""
        super().__init__(serialize_hook=serialize_hook, deserialize_hook=deserialize_hook)
        self._data = self._validate(request)

    def __iter__(self):
        yield self

    def __bool__(self):
        return self._valid_flag


class JSONRPCSingleRequest(JSONRPCBaseRequest):
    """ Main object wrapper for JSON-RPC request """

    result = None
    _data = {}
    _notification_flag = None

    REQUIRED_FIELDS = {"jsonrpc", "method"}
    POSSIBLE_FIELDS = {"jsonrpc", "method", "params", "id"}

    @property
    def args(self):
        """ Request args
        :rtype: tuple
        """
        return tuple(self.params) if isinstance(self.params, (list, tuple)) else ()

    @property
    def kwargs(self):
        """ Request kwargs
        :rtype: dict
        """
        return self.params if isinstance(self.params, dict) else {}

    @property
    def json(self):
        """ Serialized request, ready for sending
        :return: JSON-RPC request
        :rtype: str
        """
        return self.serialize(self.data)

    @property
    def data(self):
        """ Deserialized request
        :rtype: dict
        """
        data = {'jsonrpc': '2.0', 'method': self.method}
        if self.params is not None:
            data['params'] = self.params
        if not self.is_notification:
            data['id'] = self.id
        return data

    @data.setter
    def data(self, value):
        """ Set new data for request object (replaces old)
        :type value: dict
        """
        self._data = self._validate(value)

    @property
    def method(self):
        """ Request method name
        :rtype: str
        """
        return self._data.get('method')

    @property
    def params(self):
        """ Request params
        :rtype: tuple or dict or None
        """
        return self._data.get('params')

    @property
    def id(self):
        """ Request ID
        :rtype: str or int
        """
        return self._data.get('id')

    @property
    def is_notification(self):
        """ Notification flag. If Request object is notification, server never sends reply.
        If self.id is not set, flag sets automatically
        :rtype: bool
        """
        return 'id' not in self._data or self._notification_flag

    @is_notification.setter
    def is_notification(self, value):
        """ Set notification flag
        :type value: bool
        """
        self._notification_flag = bool(value)

    def process(self, dispatcher):
        """ Process request with method taken from dispatcher registry
        :type dispatcher: Dispatcher
        :rtype: JSONRPCSingleResponse or None
        """
        output = None
        try:
            method = dispatcher[self.method]
        except KeyError:
            output = JSONRPCMethodNotFound().as_response()
        else:
            try:
                result = method(*self.args, **self.kwargs)
            except TypeError:
                output = JSONRPCInvalidParams().as_response()
            except Exception as e:
                data = {'type': e.__class__.__name__, 'message': str(e)}
                output = JSONRPCServerError(data=data).as_response()
            else:
                output = JSONRPCSingleResponse(
                    result,
                    request=self,
                    serialize_hook=self.serialize_hook,
                    deserialize_hook=self.deserialize_hook
                )
        finally:
            if not self.is_notification:
                return output

    def _parse(self, string):
        try:
            data = self.deserialize(string)
        except:
            raise JSONRPCParseException("Cannot deserialize!")
        else:
            if isinstance(data, (list, tuple)):
                raise JSONRPCMultipleRequestException
            return data

    def _validate(self, raw_data):
        """ Validates request data
        :param raw_data: request
        :type raw_data: str or dict
        :return: Cleaned data
        :rtype: dict
        :raise JSONRPCInvalidRequestException:
        """
        self._valid_flag = False

        if isinstance(raw_data, str):
            data = self._parse(raw_data)
        elif isinstance(raw_data, (list, tuple)):
            raise JSONRPCMultipleRequestException
        elif isinstance(raw_data, dict):
            data = raw_data
        else:
            raise JSONRPCInvalidRequestException("Request must be str or dict, not {0}".format(type(raw_data)))

        if not data:
            raise JSONRPCInvalidRequestException("Empty data!")

        if not self.REQUIRED_FIELDS <= set(data.keys()) <= self.POSSIBLE_FIELDS:
            raise JSONRPCInvalidRequestException(
                'Invalid request. Extra fields: {0}, Missed fields: {1}'
                .format(set(data.keys()) - self.POSSIBLE_FIELDS, self.REQUIRED_FIELDS - set(data.keys()))
            )

        if data['jsonrpc'] != '2.0':
            raise JSONRPCInvalidRequestException('"jsonrpc" field MUST be set to "2.0"')

        if not isinstance(data['method'], str):
            raise JSONRPCInvalidRequestException('"method" should be str or dict, not {0}'.format(type(data['method'])))

        if data['method'].startswith('rpc.'):
            raise JSONRPCInvalidRequestException(
                'Method names that begin with the word rpc followed by a period character (U+002E or ASCII 46) '
                'are reserved for rpc-internal methods and extensions and MUST NOT be used for anything else.'
            )

        if 'id' in data and not isinstance(data['id'], (str, int)):
            raise JSONRPCInvalidRequestException('"id" must be string or integer, not {0}'.format(type(data['id'])))

        if 'params' in data and not isinstance(data['params'], (tuple, list, dict)):
            raise JSONRPCInvalidRequestException(
                '"params" should be tuple, list or dict, not {0}'
                .format(type(data['params']))
            )

        self._notification_flag = True if 'id' not in data else False
        self._valid_flag = True
        return data


class JSONRPCBatchRequest(JSONRPCBaseRequest):
    """ Batch list of JSON-RPC 2.0 Request """

    _data = []

    def __iter__(self):
        return iter(self._data)

    def __len__(self):
        return len(self._data)

    def __getitem__(self, item):
        return self._data.__getitem__(item)

    @property
    def json(self):
        return self.serialize([request.data for request in self])

    def process(self, dispatcher):
        responses = list(filter(None, [request.process(dispatcher) for request in self]))
        if responses:
            return JSONRPCBatchResponse(responses, serialize_hook=self.serialize_hook)

    def _validate(self, raw_data):
        self._valid_flag = False
        data = []
        if not raw_data:
            raise JSONRPCInvalidRequestException("Empty data!")

        if isinstance(raw_data, (list, tuple, JSONRPCSingleRequest)):
            for item in raw_data:
                if isinstance(item, dict):
                    data.append(JSONRPCSingleRequest(item))
                elif isinstance(item, JSONRPCSingleRequest):
                    data.append(item)
                else:
                    raise JSONRPCInvalidRequestException(
                        "Request must be dict, or JSONRPCSingleRequest instance, not {0}"
                        .format(type(raw_data))
                    )
        elif isinstance(raw_data, str):
            raw_data = self.deserialize(raw_data)
            if isinstance(raw_data, list):
                data = [JSONRPCSingleRequest(item) for item in data]
        else:
            raise JSONRPCInvalidRequestException(
                "Requests must be list, tuple, JSONRPCSingleRequest or str, not {0}"
                .format(type(raw_data))
            )

        self._valid_flag = True
        return data



import unittest

from mock import MagicMock

from jsonrpc.manager import JSONRPCResponseManager
from jsonrpc.request import JSONRPCBatchRequest, JSONRPCSingleRequest
from jsonrpc.response import JSONRPCBatchResponse, JSONRPCSingleResponse


class TestJSONRPCResponseManager(unittest.TestCase):
    def setUp(self):
        def raise_(e):
            raise e

        self.long_time_method = MagicMock()
        self.dispatcher = {
            "add": sum,
            "list_len": len,
            "101_base": lambda **kwargs: int("101", **kwargs),
            "error": lambda: raise_(Exception("error_explanation")),
            "long_time_method": self.long_time_method,
            "echo": lambda x: x,
        }
        self.manager = JSONRPCResponseManager()

    def test_returned_type_response(self):
        request = JSONRPCSingleRequest({'jsonrpc': '2.0', 'method': 'add', 'params': [], 'id': 0})
        response = self.manager.handle(request.json, self.dispatcher)
        self.assertIsInstance(response, JSONRPCSingleResponse)

    def test_returned_type_butch_response(self):
        request = JSONRPCBatchRequest([{'jsonrpc': '2.0', 'method': 'add', 'params': [], 'id': 0}])
        response = self.manager.handle(request.json, self.dispatcher)
        self.assertIsInstance(response, JSONRPCBatchResponse)

    def test_parse_error(self):
        req = '{"jsonrpc": "2.0", "method": "foobar, "params": "bar", "baz]'
        response = self.manager.handle(req, self.dispatcher)
        self.assertIsInstance(response, JSONRPCSingleResponse)
        self.assertEqual(response.error["message"], "Parse error")
        self.assertEqual(response.error["code"], -32700)

    def test_echo_request(self):
        req = '{"jsonrpc": "2.0", "method": "echo", "params": ["foo"], "id": 1}'
        response = self.manager.handle(req, self.dispatcher)
        self.assertIsInstance(response, JSONRPCSingleResponse)
        self.assertEqual(response.result, "foo")
        req = '{"jsonrpc": "2.0", "method": "echo", "params": [["foo", "bar"]], "id": 1}'
        response = self.manager.handle(req, self.dispatcher)
        self.assertIsInstance(response, JSONRPCSingleResponse)
        self.assertEqual(response.result, ["foo", "bar"])

    def test_invalid_request(self):
        req = '{"jsonrpc": "2.0", "method": 1, "params": "bar"}'
        response = self.manager.handle(req, self.dispatcher)
        self.assertIsInstance(response, JSONRPCSingleResponse)
        self.assertEqual(response.error["message"], "Invalid Request")
        self.assertEqual(response.error["code"], -32600)
        req = '1'
        response = self.manager.handle(req, self.dispatcher)
        self.assertIsInstance(response, JSONRPCSingleResponse)
        self.assertEqual(response.error["message"], "Invalid Request")
        self.assertEqual(response.error["code"], -32600)

    def test_method_not_found(self):
        request = JSONRPCSingleRequest({'jsonrpc': '2.0', 'method': 'does_not_exist', 'params': [], 'id': 0})
        response = self.manager.handle(request.json, self.dispatcher)
        self.assertIsInstance(response, JSONRPCSingleResponse)
        self.assertEqual(response.error["message"], "Method not found")
        self.assertEqual(response.error["code"], -32601)

    def test_invalid_params(self):
        request = JSONRPCSingleRequest({'jsonrpc': '2.0', 'method': 'add', 'params': {"a": 0}, 'id': 0})
        response = self.manager.handle(request.json, self.dispatcher)
        self.assertIsInstance(response, JSONRPCSingleResponse)
        self.assertEqual(response.error["message"], "Invalid params")
        self.assertEqual(response.error["code"], -32602)

    def test_server_error(self):
        request = JSONRPCSingleRequest({'jsonrpc': '2.0', 'method': 'error', 'id': 0})
        response = self.manager.handle(request.json, self.dispatcher)
        self.assertIsInstance(response, JSONRPCSingleResponse)
        self.assertEqual(response.error["message"], "Server error")
        self.assertEqual(response.error["code"], -32000)
        self.assertEqual(response.error["data"], {
            "type": "Exception",
            "message": 'error_explanation',
        })

    def test_notification_calls_method(self):
        request = JSONRPCSingleRequest({'jsonrpc': '2.0', 'method': 'long_time_method'})
        response = self.manager.handle(request.json, self.dispatcher)
        self.assertEqual(response, None)
        self.long_time_method.assert_called_once_with()

    def test_notification_does_not_return_error_does_not_exist(self):
        request = JSONRPCSingleRequest({'jsonrpc': '2.0', 'method': 'does_not_exist'})
        response = self.manager.handle(request.json, self.dispatcher)
        self.assertEqual(response, None)

    def test_notification_does_not_return_error_invalid_params(self):
        request = JSONRPCSingleRequest({'jsonrpc': '2.0', 'method': 'does_not_exist', 'params': {"a": 0}})
        response = self.manager.handle(request.json, self.dispatcher)
        self.assertEqual(response, None)

    def test_notification_does_not_return_error(self):
        request = JSONRPCSingleRequest({'jsonrpc': '2.0', 'method': 'error'})
        response = self.manager.handle(request.json, self.dispatcher)
        self.assertEqual(response, None)



""" Utility functions for package."""
from datetime import datetime, date, time, timedelta, tzinfo


class FixedOffset(tzinfo):
    """Fixed offset in minutes east from UTC."""

    def __init__(self, offset):
        self.__offset = timedelta(seconds=offset)

    def utcoffset(self, dt):
        return self.__offset

    def tzname(self, dt):
        return 'TZ offset: {secs} hours'.format(secs=self.__offset)

    def dst(self, dt):
        return timedelta(0)


def json_datetime_default(o):
    """Encoder for date/time/datetime objects.
    Usage: json.dumps(object, default=json_datetime_default)

    :type o: datetime | date | time
    :rtype: dict
    :raises: TypeError
    """

    if type(o) == date:
        return {'__date__': [o.year, o.month, o.day]}

    if isinstance(o, time):
        res = {'__time__': [o.hour, o.minute, o.second, o.microsecond]}
        if o.tzinfo is not None:
            res['__tzshift__'] = o.utcoffset().seconds
        return res

    if isinstance(o, datetime):
        res = {'__datetime__': [o.year, o.month, o.day, o.hour, o.minute, o.second, o.microsecond]}
        if o.tzinfo is not None:
            res['__tzshift__'] = o.utcoffset().seconds
        return res

    raise TypeError


def json_datetime_hook(dictionary):
    """JSON object_hook function for decoding date/time/datetime objects.
    Usage: json.loads(object, object_hook=json_datetime_hook)

    :type dictionary: dict
    :rtype: datetime | date | time
    """

    if '__date__' in dictionary:
        return date(*dictionary['__date__'])

    if '__time__' in dictionary:
        res = time(*dictionary['__time__'])
        if '__tzshift__' in dictionary:
            res = res.replace(tzinfo=FixedOffset(dictionary['__tzshift__']))
        return res

    if '__datetime__' in dictionary:
        res = datetime(*dictionary['__datetime__'])
        if '__tzshift__' in dictionary:
            res = res.replace(tzinfo=FixedOffset(dictionary['__tzshift__']))
        return res

    return dictionary



#!/usr/bin/env python3
import os
from setuptools import setup, find_packages
from jsonrpc import version


def read(fname):
    try:
        return open(os.path.join(os.path.dirname(__file__), fname)).read()
    except IOError:
        return ""

setup(
    name="json-rpc-3",
    version=version,
    packages=find_packages(),
    test_suite="nose.collector",
    tests_require=["nose", "mock"],
    author='see AUTHORS',
    maintainer='Orhideous',
    maintainer_email='orhideous@gmail.com',
    url="https://github.com/Orhideous/json-rpc",
    description="Pure Python 3 JSON-RPC 2.0 transport realisation",
    long_description=read('README.rst'),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    keywords=["json", "rpc", "json-rpc", "transport"],
    license="MIT",
)



""" Example of json-rpc usage with Wergzeug and requests.

NOTE: there are no Werkzeug and requests in dependencies of json-rpc.
NOTE: server handles all url paths the same way (there are no different urls).
"""

from werkzeug.wrappers import Request, Response
from werkzeug.serving import run_simple

from jsonrpc import JSONRPCResponseManager, dispatcher

from api import dispatcher


manager = JSONRPCResponseManager()


@Request.application
def application(request):
    response = manager.handle(request.get_data(cache=False, as_text=True), dispatcher)
    return Response(response.json, mimetype='application/json')


if __name__ == '__main__':
    run_simple('localhost', 4000, application)



""" Test utility functionality."""
from datetime import datetime, date, time, timedelta
import json
import unittest

from jsonrpc.utils import json_datetime_hook, FixedOffset, json_datetime_default


class TestDatetimeEncoderDecoder(unittest.TestCase):
    """ Test DatetimeEncoder and json_datetime_hook functionality."""

    def test_fixed_offset_class(self):
        tzinfo = FixedOffset(3600)
        dt = datetime.now(tz=tzinfo)
        self.assertEqual(dt.dst(), timedelta(0))
        self.assertEqual(dt.tzname(), "TZ offset: 1:00:00 hours")

    def test_incorrect(self):
        obj = NotImplementedError

        with self.assertRaises(TypeError):
            json.dumps(obj)

    def test_datetime(self):
        obj = datetime.now()
        incorrect = NotImplementedError

        with self.assertRaises(TypeError):
            json.dumps(obj)

        with self.assertRaises(TypeError):
            json.dumps(incorrect, default=json_datetime_default)

        string = json.dumps(obj, default=json_datetime_default)

        self.assertEqual(obj, json.loads(string, object_hook=json_datetime_hook))

    def test_date(self):
        obj = date.today()

        string = json.dumps(obj, default=json_datetime_default)

        self.assertEqual(obj, json.loads(string, object_hook=json_datetime_hook))

    def test_time(self):
        obj = datetime.now().time()
        obj_tz = datetime.now().time().replace(tzinfo=FixedOffset(3600))

        string = json.dumps(obj, default=json_datetime_default)
        string_tz = json.dumps(obj_tz, default=json_datetime_default)

        self.assertEqual(obj, json.loads(string, object_hook=json_datetime_hook))
        self.assertEqual(obj_tz, json.loads(string_tz, object_hook=json_datetime_hook))
        self.assertIsNotNone(json.loads(string_tz, object_hook=json_datetime_hook).tzinfo)

    def test_complex(self):
        obj = {'id': '1', 'params': (datetime(2014, 6, 17, 9, 38, 39, 911853),), 'jsonrpc': 2.0, 'method': 'w'}
        json.dumps(obj, default=json_datetime_default)

    def test_skip_nondt_obj(self):
        obj = {'__weird__': True}
        string = json.dumps(obj, default=json_datetime_default)
        self.assertEqual(obj, json.loads(string, object_hook=json_datetime_hook))

    def test_datetime_tzinfo(self):
        obj = datetime.now().replace(tzinfo=FixedOffset(3600))

        with self.assertRaises(TypeError):
            json.dumps(obj)

        string = json.dumps(obj, default=json_datetime_default)

        self.assertEqual(obj, json.loads(string, object_hook=json_datetime_hook))



from jsonrpc import dispatcher


def dict_to_list(dictionary):
    return list(dictionary.items())


@dispatcher.add_method
def simple_add(first=0, **kwargs):
    return first + kwargs["second"]


def echo_with_long_name(msg):
    return msg

dispatcher.add_method(echo_with_long_name, name='echo')

dispatcher['subtract'] = lambda a, b: a - b
dispatcher['dict_to_list'] = dict_to_list
